"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const loader_utils_1 = require("loader-utils");
const twing_1 = require("twing");
const node_visitor_1 = require("./node-visitor");
const sha256 = require('crypto-js/sha256');
const hex = require('crypto-js/enc-hex');
const validateOptions = require('schema-utils');
const optionsSchema = {
    type: 'object',
    properties: {
        environmentModulePath: {
            type: 'string'
        },
        renderContext: {
            type: 'object'
        }
    },
    required: [
        'environmentModulePath'
    ],
    additionalProperties: false
};
class PathSupportingArrayLoader extends twing_1.TwingLoaderArray {
    getSourceContext(name) {
        let source = super.getSourceContext(name);
        return new twing_1.TwingSource(source.getCode(), source.getName(), name);
    }
}
function default_1(source) {
    const options = loader_utils_1.getOptions(this);
    validateOptions(optionsSchema, options, 'Twing loader');
    let environmentModulePath = options.environmentModulePath;
    let renderContext = options.renderContext;
    this.addDependency(environmentModulePath);
    // we don't want to reuse an eventual initialized environment
    delete require.cache[require.resolve(environmentModulePath)];
    let environment = require(environmentModulePath);
    if (renderContext === undefined) {
        let parts = [
            `const {cache, loader, getEnvironment} = require('${require.resolve('./runtime')}');`,
            `const env = getEnvironment(require('${environmentModulePath}'));`
        ];
        let nodeVisitor;
        nodeVisitor = new node_visitor_1.NodeVisitor();
        nodeVisitor.fromPath = this.resourcePath;
        environment.addNodeVisitor(nodeVisitor);
        Reflect.set(environment, 'getTemplateClass', function (name, index = null, from = null) {
            let hash;
            const HASH_PREFIX = '__HASHED__';
            if (name.startsWith(HASH_PREFIX)) {
                hash = name;
            }
            else {
                hash = `${HASH_PREFIX}${hex.stringify(sha256(name))}`;
            }
            return hash + (index === null ? '' : '_' + index);
        });
        let className = environment.getTemplateClass(this.resourcePath);
        let sourceContext = new twing_1.TwingSource(source, className);
        let precompiledTemplate = environment.compile(environment.parse(environment.tokenize(sourceContext)));
        parts.push(`cache.write('${className}', (() => {let module = {
    exports: undefined
};

${precompiledTemplate}

return module.exports;})());
`);
        parts.push(`loader.addTemplateKey('${className}', '${className}');`);
        for (let foundTemplateName of nodeVisitor.foundTemplateNames) {
            parts.push(`require('${foundTemplateName}');`);
        }
        parts.push(`
let template = env.loadTemplate('${className}');

module.exports = function(context = {}) {
    return template.render(context);
};`);
        return parts.join('\n');
    }
    else {
        environment.setLoader(new twing_1.TwingLoaderChain([
            new PathSupportingArrayLoader(new Map([
                [this.resourcePath, source]
            ])),
            environment.getLoader()
        ]));
        environment.on('template', (name, from) => {
            this.addDependency(environment.getLoader().resolve(name, from));
        });
        return `module.exports = ${JSON.stringify(environment.render(this.resourcePath, renderContext))};`;
    }
}
exports.default = default_1;
;
