"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const twing_1 = require("twing");
const fs_1 = require("fs");
class NodeVisitor extends twing_1.TwingBaseNodeVisitor {
    get foundTemplateNames() {
        return this._foundTemplateNames;
    }
    set fromPath(fromPath) {
        this._sourceContext = new twing_1.TwingSource('', fromPath, fromPath);
        this._foundTemplateNames = [];
    }
    doEnterNode(node, env) {
        return node;
    }
    doLeaveNode(node, env) {
        // include function
        if ((node.getType() === twing_1.TwingNodeType.EXPRESSION_FUNCTION) && (node.getAttribute('name') === 'include')) {
            this.processExpressionNode(node.getNode('arguments').getNode(0), env);
        }
        // import and include tags
        if ((node.getType() === twing_1.TwingNodeType.IMPORT) || (node.getType() === twing_1.TwingNodeType.INCLUDE)) {
            this.processExpressionNode(node.getNode('expr'), env);
        }
        // extends and embed tags
        if ((node.getType() === twing_1.TwingNodeType.MODULE) && node.hasNode('parent')) {
            this.processExpressionNode(node.getNode('parent'), env);
        }
        return node;
    }
    processExpressionNode(node, env) {
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_ARRAY) {
            for (let [index, constantNode] of node.getNodes()) {
                if (index % 2) {
                    this.processExpressionNode(constantNode, env);
                }
            }
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_CONDITIONAL) {
            let expr2 = node.getNode('expr2');
            let expr3 = node.getNode('expr3');
            this.processExpressionNode(expr2, env);
            this.processExpressionNode(expr3, env);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_CONSTANT) {
            node.setAttribute('value', this.pushValue(node.getAttribute('value'), env));
        }
    }
    pushValue(value, env) {
        if (env.getLoader().exists(value, this._sourceContext)) {
            let resolvedPath = env.getLoader().resolve(value, this._sourceContext);
            if (fs_1.existsSync(resolvedPath)) {
                if (!this._foundTemplateNames.includes(resolvedPath)) {
                    this._foundTemplateNames.push(resolvedPath);
                }
                return env.getTemplateClass(resolvedPath);
            }
        }
        return value;
    }
    getPriority() {
        return 0;
    }
}
exports.NodeVisitor = NodeVisitor;
