"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const twing_1 = require("twing");
class CustomCache extends twing_1.TwingCacheNull {
    constructor() {
        super();
        /**
         * @type {Map<string, string>}
         */
        this._templates = new Map();
        this.TwingCacheInterfaceImpl = this;
    }
    /**
     * Generates a cache key for the given template class name.
     *
     * @param {string} name The template name
     * @param {string} className The template class name
     *
     * @return string
     */
    generateKey(name, className) {
        return className;
    }
    /**
     * Writes the compiled template to cache.
     *
     * @param {string} key The cache key
     * @param {string} content The template representation as a PHP class
     */
    write(key, content) {
        if (typeof content === 'string') {
            content = new Function(`let module = {
    exports: undefined
};

${content}

return module.exports;
`);
        }
        this._templates.set(key, content);
    }
    ;
    /**
     * Loads a template from the cache.
     *
     * @param {string} key The cache key
     */
    load(key) {
        if (!this._templates.has(key)) {
            return super.load(key);
        }
        return this._templates.get(key);
    }
    /**
     * Returns the modification timestamp of a key.
     *
     * @param {string} key The cache key
     *
     * @returns {number}
     */
    getTimestamp(key) {
        return null;
    }
}
/**
 * Custom loader used to provide a predictable cache key.
 */
class CustomLoader extends twing_1.TwingLoaderArray {
    constructor(templatesMap = new Map()) {
        super({});
        this._templatesMap = templatesMap;
    }
    getCacheKey(name) {
        return name;
    }
    addTemplateKey(name, key) {
        this._templatesMap.set(name, key);
    }
    /**
     * Needed because TwingLoaderChain::getCacheKey calls exists() on the loaders.
     */
    exists(name) {
        return this._templatesMap.has(name);
    }
}
let environment;
exports.cache = new CustomCache();
exports.loader = new CustomLoader();
exports.getEnvironment = (env) => {
    if (!environment) {
        env.setLoader(new twing_1.TwingLoaderChain([exports.loader, env.getLoader()]));
        env.setCache(exports.cache);
        Reflect.set(env, 'getTemplateClass', (name) => name);
        environment = env;
    }
    return environment;
};
